@@ -20,6 +20,7 @@ from typing import (
     Dict,
     Iterable,
     List,
+    MutableMapping,
     Optional,
     Tuple,
     TYPE_CHECKING,
@@ -27,8 +28,6 @@ from typing import (
 )
 import attr
 
-from markdown_it.utils import AttrDict
-
 if TYPE_CHECKING:
     from markdown_it import MarkdownIt
 
@@ -36,21 +35,19 @@ if TYPE_CHECKING:
 class StateBase:
     srcCharCode: Tuple[int, ...]
 
-    def __init__(self, src: str, md: "MarkdownIt", env: AttrDict):
+    def __init__(self, src: str, md: "MarkdownIt", env: MutableMapping):
         self.src = src
         self.env = env
         self.md = md
 
     @property
-    def src(self):
+    def src(self) -> str:
         return self._src
 
     @src.setter
-    def src(self, value):
+    def src(self, value: str) -> None:
         self._src = value
-        self.srcCharCode = (
-            tuple(ord(c) for c in self.src) if self.src is not None else ()
-        )
+        self.srcCharCode = tuple(ord(c) for c in self.src)
 
 
 # The first positional arg is always a subtype of `StateBase`. Other
@@ -116,7 +113,7 @@ class Ruler:
         if index == -1:
             raise KeyError(f"Parser rule not found: {ruleName}")
         self.__rules__[index].fn = fn
-        self.__rules__[index].alt = options["alt"] or []
+        self.__rules__[index].alt = options.get("alt", [])
         self.__cache__ = None
 
     def before(self, beforeName: str, ruleName: str, fn: RuleFunc, options=None):
