@@ -3,3 +3,5 @@ from .normalize import normalize  # noqa: F401
 from .block import block  # noqa: F401
 from .inline import inline  # noqa: F401
 from .replacements import replace  # noqa: F401
+from .smartquotes import smartquotes  # noqa: F401
+from .linkify import linkify  # noqa: F401
