@@ -9,7 +9,7 @@ NEWLINES_RE = re.compile(r"\r\n?|\n")
 NULL_RE = re.compile(r"\0")
 
 
-def normalize(state: StateCore):
+def normalize(state: StateCore) -> None:
 
     # Normalize newlines
     string = NEWLINES_RE.sub("\n", state.src)
